/********************************************************
 *  ██████╗  ██████╗████████╗██╗
 * ██╔════╝ ██╔════╝╚══██╔══╝██║
 * ██║  ███╗██║        ██║   ██║
 * ██║   ██║██║        ██║   ██║
 * ╚██████╔╝╚██████╗   ██║   ███████╗
 *  ╚═════╝  ╚═════╝   ╚═╝   ╚══════╝
 * Geophysical Computational Tools & Library (GCTL)
 *
 * Copyright (c) 2022  Yi Zhang (yizhang-geo@zju.edu.cn)
 *
 * GCTL is distributed under a dual licensing scheme. You can redistribute 
 * it and/or modify it under the terms of the GNU Lesser General Public 
 * License as published by the Free Software Foundation, either version 2 
 * of the License, or (at your option) any later version. You should have 
 * received a copy of the GNU Lesser General Public License along with this 
 * program. If not, see <http://www.gnu.org/licenses/>.
 * 
 * If the terms and conditions of the LGPL v.2. would prevent you from using 
 * the GCTL, please consider the option to obtain a commercial license for a 
 * fee. These licenses are offered by the GCTL's original author. As a rule, 
 * licenses are provided "as-is", unlimited in time for a one time fee. Please 
 * send corresponding requests to: yizhang-geo@zju.edu.cn. Please do not forget 
 * to include some description of your company and the realm of its activities. 
 * Also add information on how to contact you by electronic and paper mail.
 ******************************************************/

#include "../lib/mesh.h"

int main(int argc, char *argv[])
{
	gctl::regular_grid rgd;
	rgd.init("grid-1", "null", 4, 4, 0.0, 0.0, 1.0, 1.0);

	rgd.add_data(gctl::NodeData, gctl::Scalar, "data-1", 2.5);
	rgd.add_data(gctl::ElemData, gctl::Scalar, "data-2", 1.2, false);
	rgd.add_data(gctl::NodeData, gctl::Scalar, "data-3", 1.0);
	rgd.show_info();

	gctl::meshdata data2 = rgd.get_data("data-2");
	std::cout << "data name: " << data2.name_ << std::endl;
	data2.datval_.show();

	gctl::meshdata data3 = rgd.get_data("data-3");
	std::cout << "data name: " << data3.name_ << std::endl;
	data3.datval_.show();

	rgd.diff("data-4", "data-1", "data-3");

	rgd.remove_data("data-1");
	rgd.show_info();
	rgd.save_gmsh_withdata("ex1_out", gctl::OverWrite, gctl::NotPacked);
	return 0;
}